import json
import pulumi
import pulumi_aws

from pulumi.output import Output

from pydantic.dataclasses import dataclass, ConfigDict
from typing import  Optional, List, Mapping

from theia import ResourceOptions
from theia.aws.containers.ECS.cluster import ECSCluster, ECSClusterVpcArgs
from theia.aws.containers.ECS.ecs_service import create_ecs_service
from theia.aws.containers.ECS.loadbalancer import ECSLoadBalancer
from theia.aws.containers.ECS.task_definition import TaskDefinition
from theia.aws.containers.ECS.policies.policy import ecs_exec_role_policy, assume_role_policy
from theia.aws.security.iam import InlinePolicy, Role
from theia.utils.logger import Logger


logger = Logger("eks")



@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class ECSSettings:
    subnet_zones: Optional[List[str]] = None
    public_access_cidrs: Optional[List[str | Output[str]]] = None
    default_addons_to_remove: Optional[List[str]] = None



class ECS(pulumi.ComponentResource):
    def __init__(
        self,
        cpu:int = None,
        memory:int = None,
        network_mode:str = "awsvpc",
        requires_compatibilities: List[str] = ["FARGATE"],
        resource_name: str = None,
        subnet_ids: Mapping[str, str]= None,
        cluster_vpc_args: Optional[ECSClusterVpcArgs] = ECSClusterVpcArgs(),
        security_group: Output[pulumi_aws.ec2.SecurityGroup] = None,
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None
    ) -> None:
        super().__init__("theia:aws:ecs", resource_name, None, opts)
        
        account_id = pulumi_aws.get_caller_identity().account_id

        region:str = pulumi_aws.get_region().name
        
        self.execution_role: pulumi_aws.iam.Role = Role(
            resource_name=resource_name,
            description="role for running taskdefinitions on ECS",
            assume_role_policy=json.dumps(assume_role_policy()),
            inline_policies=[InlinePolicy(
                name=f"{resource_name}",
                policy=json.dumps(ecs_exec_role_policy(region=region,account_id=account_id))
            )],
            tags=tags,
            opts=ResourceOptions(parent=self)
        )
                        
        self.cluster = ECSCluster(
            resource_name=resource_name,
            subnet_ids=subnet_ids,
            cluster_vpc_args=cluster_vpc_args,
            tags=tags,opts=opts
        )

        self.LB = ECSLoadBalancer(resource_name=resource_name,subnet_ids=subnet_ids,vpc_id=cluster_vpc_args.vpc_id,tags=tags)

        self.task_definition = TaskDefinition(
            cpu=cpu,
            resource_name=resource_name, 
            memory=memory, 
            network_mode= network_mode, 
            region=region,
            requires_compatibilities=requires_compatibilities,
            exec_role_arn=self.execution_role.arn,
            load_balancer=self.LB.load_balancer, 
            target_group=self.LB.target_group, 
            subnet_ids=subnet_ids
        )

        self.ecs_service = create_ecs_service(resource_name=resource_name,
            cluster=self.cluster,
            task_definition=self.task_definition.task_definition,
            subnet_ids=subnet_ids,
            target_group=self.LB.target_group,
            security_groups=[security_group]
        )

        self.register_outputs({})