import pulumi
from pulumi import Input, InputType

from pulumi_aws import athena

from typing import Optional, Mapping, Literal, List, Any
from pydantic.dataclasses import dataclass, ConfigDict
from theia.utils.errors import TheiaConfigurationError

from theia import ResourceOptions
from theia._registry import StackRegistry, GlobalTags
from theia.definitions import THEIA_VERSION
from theia.aws.network.eip import Eip
from pulumi import Output
import time
import re


@dataclass
class WorkgroupArgs:
    description: Optional[str] = None
    force_destroy: Optional[bool] = None
    state: Literal["ENABLED", "DISABLED"] = "ENABLED"
    tags: Optional[Mapping[str, str]] = None


class Workgroup:
    def __new__(
            cls,
            name: str,
            args: Optional[WorkgroupArgs] = None,
            opts: Optional[ResourceOptions] = None,
            ) -> athena.Workgroup:

        args_values = None
        if args:
            args_values = athena.WorkgroupArgs(
                name=name,
                description=args.description,
                force_destroy=args.force_destroy,
                state=args.state,
                tags=args.tags,
                )
        else:
            args_values = athena.WorkgroupArgs(
                name=name
                )

        workgroup = athena.Workgroup(
            name,
            args=args_values,
            opts=opts,
            )

        return workgroup


@dataclass
class DatabaseAclConfiguration:
    s3_acl_option: str


@dataclass
class DatabaseEncryptionConfiguration:
    encryption_option: Literal["SSE_S3", "SSE_KMS", "CSE_KMS"]
    kms_key: Optional[str] = None
    def __post_init__(self):
        if self.encryption_option == "SSE_KMS" or self.encryption_option == "CSE_KMS":
            if not self.kms_key:
                raise TheiaConfigurationError(
                    "KMS key ARN or ID required for encryption_option types SSE_KMS and CSE_KMS"
                )


@dataclass
class DatabaseArgs:
    force_destroy: Optional[bool] = True
    expected_bucket_owner: Optional[str] = None
    comment: Optional[str] = None
    properties: Optional[Mapping[str, str]] = None
    # acl_configuration: Optional[DatabaseAclConfigurationArgs] = None
    # encryption_configuration: Optional[DatabaseEncryptionConfigurationArgs] = None


class Database:
    def __new__(
            cls,
            name: str,
            bucket: str,
            args: Optional[DatabaseArgs] = None,
            opts: Optional[ResourceOptions] = None,
            ) -> athena.Database:
        
        args_values=None
        if args:
            args_values = athena.DatabaseArgs(
                name=name,
                bucket=bucket,
                force_destroy=args.force_destroy,
                expected_bucket_owner=args.expected_bucket_owner,
                comment=args.comment,
                # acl_configuration=args.acl_configuration,
                # encryption_configuration=args.encryption_configuration,
                )
        else:
            args_values = athena.DatabaseArgs(
                name=name,
                bucket=bucket,
                )

        db = athena.Database(
            name+"---"+bucket,
            args=args_values,
            opts=opts,
            )

        pulumi.export("Athena-Database---", db)

        return db


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class NamedQueryArgs:
    database: str | Output[str]
    description: Optional[str] = None
    workgroup: Optional[Output[str] | str] = "primary"


class NamedQuery:
    def __new__(
            cls,
            name: str,
            query: str,
            args: NamedQueryArgs,
            opts: Optional[ResourceOptions] = None,
            ) -> athena.NamedQuery:
        
        args_values = athena.NamedQueryArgs(
            name=name,
            query=query,
            database=args.database,
            description=args.description,
            workgroup=args.workgroup,
            )

        query = athena.NamedQuery(
            name+"---"+args.workgroup,
            args=args_values,
            opts=opts,
            )
        
        pulumi.export("Athena-NamedQuery---", query)
        
        return query