import pulumi

from pulumi.output import Output
from pulumi_aws import elasticache

from typing import Optional, List, Mapping
from pydantic.dataclasses import dataclass, ConfigDict

from theia import ResourceOptions
from theia.definitions import THEIA_VERSION
from theia._registry import StackRegistry, GlobalTags
from theia.utils.errors import TheiaConfigurationError
from theia.aws.database.parameter_group import ParameterGroup


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class RedisSettings:
    apply_immediately: Optional[bool] = False
    auth_token: Optional[str] = None
    auto_minor_version_upgrade: Optional[bool] = False
    automatic_failover_enabled: Optional[bool] = False
    final_snapshot_identifier: Optional[str] = None
    maintenance_window: Optional[str] = None
    multi_az_enabled: Optional[bool] = False
    node_type: Optional[str] = None
    num_cache_clusters: Optional[int] = 1
    num_node_groups: Optional[int] = None
    parameter_group_name: Optional[str] = None
    port: Optional[int] = None
    replicas_per_node_group: Optional[int] = None
    vpc_security_group_ids: Optional[List[str | Output[str]]] = None

    def __post_init__(self):
        if self.automatic_failover_enabled and self.num_cache_clusters <= 1:
            raise TheiaConfigurationError(
                "automatic failover enabled but number of cache clusters is not greater than 1"
            )

        if self.multi_az_enabled and (self.num_cache_clusters <= 1 or not self.automatic_failover_enabled):
            raise TheiaConfigurationError(
                "multi az was enabled but num cache clusters or automatic failover not configured correctly"
            )


class Redis(pulumi.ComponentResource):
    def __init__(
        self,
        resource_name: str,
        engine_version: str,
        family: str,
        description: Optional[str],
        parameter_groups: Optional[ParameterGroup] = None,
        subnet_ids: Optional[List[str]] = None,
        settings: Optional[RedisSettings] = RedisSettings(),
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
    ) -> None:
        super().__init__("theia:aws:elasticache", resource_name, None, opts)

        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        if parameter_groups:
            parameters = []
            for param in parameter_groups.params:
                parameters.append(elasticache.ParameterGroupParameterArgs(name=param.name, value=param.value))
            self.redis_pg = elasticache.ParameterGroup(
                f"{resource_name}-redis-pg",
                description=parameter_groups.description,
                family=family,
                name=f"{resource_name}-redis-pg",
                parameters=parameters,
                opts=ResourceOptions(parent=self),
                tags=merged_tags,
            )
        else:
            parameters = [
                elasticache.ParameterGroupParameterArgs(
                    name="notify-keyspace-events",
                    value="AKE",
                )
            ]
            self.redis_pg = elasticache.ParameterGroup(
                f"{resource_name}-redis-pg",
                description=f"default parameter group for {resource_name}",
                family=family,
                name=f"{resource_name}-redis-pg",
                parameters=parameters,
                opts=ResourceOptions(parent=self),
                tags=merged_tags,
            )

        if subnet_ids:
            self.subnet_group = elasticache.SubnetGroup(
                f"{resource_name}-subnet-group",
                name=f"{resource_name}-subnet-group",
                subnet_ids=subnet_ids,
                description=f"Subnet Group for {resource_name}",
                opts=ResourceOptions(parent=self),
                tags=merged_tags,
            )

        redis_args = elasticache.ReplicationGroupArgs(
            replication_group_id=f"{resource_name}-rg",
            description=description,
            apply_immediately=settings.apply_immediately,
            auth_token=settings.auth_token,
            auto_minor_version_upgrade=settings.auto_minor_version_upgrade,
            automatic_failover_enabled=settings.automatic_failover_enabled,
            final_snapshot_identifier=settings.final_snapshot_identifier,
            maintenance_window=settings.maintenance_window,
            multi_az_enabled=settings.multi_az_enabled,
            node_type=settings.node_type,
            num_cache_clusters=settings.num_cache_clusters,
            num_node_groups=settings.num_node_groups,
            parameter_group_name=self.redis_pg.name,
            port=settings.port,
            replicas_per_node_group=settings.replicas_per_node_group,
            security_group_ids=settings.vpc_security_group_ids,
            subnet_group_name=None if not subnet_ids else self.subnet_group.name,
            transit_encryption_enabled=bool(settings.auth_token),
            engine="redis",
            engine_version=engine_version,
            tags=merged_tags,
        )
        depends = []
        depends.append(self.redis_pg)
        if subnet_ids:
            depends.append(self.subnet_group)

        self.replication_group = elasticache.ReplicationGroup(
            f"{resource_name}-rg",
            args=redis_args,
            opts=ResourceOptions(parent=self, depends_on=depends),
        )

        pulumi.export(
            f"redis---{resource_name}",
            {
                "id": self.replication_group.id,
                "arn": self.replication_group.arn,
                "primary_endpoint_address": self.replication_group.primary_endpoint_address,
            },
        )

        self.register_outputs({})

    @staticmethod
    def import_(stack: str, resource_name: str) -> elasticache.ReplicationGroup:
        stack_ref = StackRegistry.get(stack_name=stack)
        redis = stack_ref.get_output(name=f"redis---{resource_name}")
        return elasticache.ReplicationGroup.get(resource_name=resource_name, id=redis["id"])
