import pulumi
from pulumi import Input, InputType

from pulumi_aws import glue

from typing import Optional, Mapping, Literal, List, Any
from pydantic.dataclasses import dataclass, ConfigDict
from theia.utils.errors import TheiaConfigurationError

from theia import ResourceOptions
from theia._registry import StackRegistry, GlobalTags
from theia.definitions import THEIA_VERSION
from theia.aws.network.eip import Eip
from pulumi import Output
import time
import re


@dataclass
class CatalogDatabaseArgs:
    catalog_id: Optional[str] = None
    description: Optional[str] = None
    location_uri: Optional[str] = None
    parameters: Optional[Mapping[str, str]] = None
    tags: Optional[Mapping[str, str]] = None



class CatalogDatabase:
    def __new__(
            cls,
            name: str,
            args: Optional[CatalogDatabaseArgs] = None,
            opts: Optional[ResourceOptions] = None,
            ) -> glue.CatalogDatabase:
        
        # Name of the database. The acceptable characters are lowercase letters, numbers, and the underscore character

        if args:
            args_values = glue.CatalogDatabaseArgs(
                name=name,
                catalog_id=args.catalog_id,
                description=args.description,
                location_uri=args.location_uri,
                parameters=args.parameters,
                tags=args.tags,
                )
        else:
            args_values = glue.CatalogDatabaseArgs(
                name=name,
            )

        cdb = glue.CatalogDatabase(
            name,
            args=args_values,
            opts=opts,
            )
        
        pulumi.export("Glue-CatalogDatabase---"+name, cdb)

        return cdb
    
    @staticmethod
    def get(
        name: str = None,
        id: Output[str | Optional[str]] = None,
        catalog_id: Optional[str] = None,
        opts: Optional[ResourceOptions] = None,
        arn: Optional[str] = None,
        # create_table_default_permissions: Optional[Sequence[CatalogDatabaseCreateTableDefaultPermissionArgs]] = None,
        description: Optional[str] = None,
        location_uri: Optional[str] = None,
        parameters: Optional[Mapping[str, str]] = None,
        tags: Optional[Mapping[str, str]] = None,
        tags_all: Optional[Mapping[str, str]] = None,
        # target_database: Optional[CatalogDatabaseTargetDatabaseArgs] = None,
        ) -> glue.CatalogDatabase:

        if not id and (not name or not catalog_id):
            raise TheiaConfigurationError(
                "id or catalog_id need to specify"
            )
        
        if not id:
            id = catalog_id+":"+name

        resource_name = "existing-db---"+name

        cdb = glue.CatalogDatabase.get(
            resource_name=resource_name,
            id = id,
            opts=opts,
            arn=arn,
            catalog_id=catalog_id,
            description=description,
            location_uri=location_uri,
            parameters=parameters,
            tags=tags,
            tags_all=tags_all,
        )
                
        pulumi.export("Glue-CatalogDatabase---"+resource_name, cdb)

        return cdb


@dataclass
class CatalogTableArgs:
    catalog_id: Optional[str] = None
    description: Optional[str] = None
    owner: Optional[str] = None
    parameters: Optional[str] = None
    retention: Optional[str] = None
    table_type: Optional[str] = None
    view_expanded_text: Optional[str] = None
    view_original_text: Optional[str] = None

@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class CatalogTable:
    def __new__(
            cls,
            name: str,
            database_name: str,
            args: Optional[CatalogTableArgs] = None,
            opts: Optional[ResourceOptions] = None,
            ) -> glue.CatalogTable:
        
        if args:
            args_values = glue.CatalogTableArgs(
                name=name,
                database_name=database_name,
                catalog_id=args.catalog_id,
                description=args.description,
                owner=args.owner,
                parameters=args.parameters,
                retention=args.retention,
                table_type=args.table_type,
                view_expanded_text=args.view_expanded_text,
                view_original_text=args.view_original_text,
            )
        else:
            args_values = glue.CatalogTableArgs(
                name=name,
                database_name=database_name,
            )

        resource_name = name+"-"+database_name

        table = glue.CatalogTable(
            resource_name=resource_name,
            args=args_values,
            opts=opts,
        )

        pulumi.export("Glue-CatalogTable---"+resource_name, table)

        return table
    
    @staticmethod
    def get(
        name: str,
        id: Optional[str] = None,
        catalog_id: Optional[str] = None,
        database_name: Optional[str] = None,) -> glue.CatalogTable:

        if not id and (
            not catalog_id or
            not database_name or
            not name
        ):
            raise TheiaConfigurationError(
                "if not specifying 'id' then 'catalog_id' and 'database_name' required parameters"
            )

        if not id:
            id = catalog_id+":"+database_name+":"+name

        resource_name = "existing-table---"+name

        table = glue.CatalogTable.get(
            resource_name=resource_name,
            id=id,
        )

        pulumi.export("Glue-CatalogTable---"+resource_name, table)

        return table


@dataclass
class CrawlerS3TargetArgs:
    path: str
    connection_name: Optional[str] = None
    dlq_event_queue_arn: Optional[str] = None
    event_queue_arn: Optional[str] = None
    exclusions: Optional[List[str]] = None
    sample_size: Optional[int] = None
    


@dataclass
class CrawlerArgs:
    database_name: str
    role: str
    s3_targets: Optional[List[CrawlerS3TargetArgs]] = None
    classifiers: Optional[List[str]] = None
    configuration: Optional[str] = None
    description: Optional[str] = None
    schedule: Optional[str] = None
    security_configuration: Optional[str] = None
    table_prefix: Optional[str] = None
    tags: Optional[Mapping[str, str]] = None



class Crawler:
    def __new__(
            cls,
            name: str,
            args: CrawlerArgs,
            opts: Optional[ResourceOptions] = None,
        ) -> glue.Crawler:

        s3_targets = None
        if args.s3_targets:
            s3_targets = list(
                map(
                    lambda target: glue.CrawlerS3TargetArgs(
                        path=target.path,
                        connection_name=target.connection_name,
                        dlq_event_queue_arn=target.dlq_event_queue_arn,
                        event_queue_arn=target.event_queue_arn,
                        exclusions=target.exclusions,
                        sample_size=target.sample_size,
                    ),
                    args.s3_targets
                )
            )
            

        args_values = glue.CrawlerArgs(
            database_name=args.database_name,
            role=args.role,
            s3_targets=s3_targets,
            classifiers=args.classifiers,
            configuration=args.configuration,
            description=args.description,
            schedule=args.schedule,
            security_configuration=args.security_configuration,
            table_prefix=args.table_prefix,
            tags=args.tags,
        )

        crawler = glue.Crawler(
            resource_name=name,
            args=args_values,
            opts=opts,
        )

        pulumi.export("Glue-Crawler---"+name, crawler)

        return crawler
