import pulumi
from pulumi.output import Output
from pulumi_aws import cloudfront, secretsmanager
from typing import Optional, List, Mapping, Literal
from dataclasses import field
from pydantic.dataclasses import dataclass, ConfigDict
from theia import ResourceOptions
from theia.utils.logger import Logger
from theia._registry import StackRegistry, GlobalTags
from theia.definitions import THEIA_VERSION
from theia.utils.errors import TheiaConfigurationError

from Crypto.PublicKey import RSA

logger = Logger("Cloudfront")


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class DistributionSettings:
    aliases: Optional[List[str]] = None
    default_root_object: Optional[str] = None
    http_version: Optional[Literal["http1.1", "http2", "http2and3", "http3"]] = "http2"
    price_class: Optional[Literal["PriceClass_All", "PriceClass_200", "PriceClass_100"]] = None


@dataclass
class CustomErrorResponses:
    error_code: int
    error_caching_min_ttl: Optional[int] = None
    response_code: Optional[int] = None
    response_page_path: Optional[str] = None


@dataclass
class ForwardedCookies:
    forward: Optional[Literal["all", "none", "whitelist"]] = field(default_factory=lambda: "all")
    whitelisted_names: Optional[List[str]] = field(default_factory=lambda: [])

    def __post_init__(self):
        if self.forward == "whitelist" and self.whitelisted_names is None:
            raise TheiaConfigurationError("You must specify whitelisted_names when using whitelist for cookies")


@dataclass
class ForwardedValues:
    query_string: bool
    query_string_cache_keys: Optional[List[str]] = None
    cookies: Optional[ForwardedCookies] = field(default_factory=lambda: ForwardedCookies())
    headers: Optional[List[str]] = None


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class FunctionAssociations:
    event_type: Literal["viewer-request", "viewer-response", "origin-response", "origin-resquest"]
    function_type: Optional[Literal["cloudfront", "lambda"]] = "cloudfront"
    function_arn: Optional[Output[str] | str] = None
    lambda_arn: Optional[Output[str] | str] = None
    include_body: Optional[bool] = None

    def __post_init__(self):
        if self.function_type == "Lambda" and not self.lambda_arn:
            raise TheiaConfigurationError(
                "You must specify lambda_arn when using Lambda as function type for function associations"
            )
        elif not self.function_type and not self.function_arn:
            raise TheiaConfigurationError(
                "You must specify function_arn when using Lambda as function type for function associations"
            )
        elif (
            not self.function_type
            and self.function_arn
            and (self.event_type != "viewer-request" or self.event_type != "viewer-response")
        ):
            raise TheiaConfigurationError(
                "You must specify event-type as viewer-request or viewer-response for function associations or set function_type to Lambda"
            )
        elif not self.function_type and self.include_body:
            raise TheiaConfigurationError(
                "You must specify include_body when using Lambda as function type for function associations"
            )


@dataclass
class CacheBehaviour:
    target_origin_id: str
    viewer_protocol_policy: Literal["allow-all", "https-only", "redirect-to-https"]
    behaviour_type: Optional[Literal["default", "ordered"]] = "default"
    allowed_methods: Optional[List[Literal["GET", "POST", "HEAD", "DELETE", "PUT", "OPTIONS", "PATCH"]]] = field(
        default_factory=lambda: ["GET", "HEAD"]
    )
    cached_methods: Optional[List[Literal["GET", "POST", "HEAD", "DELETE", "PUT", "OPTIONS", "PATCH"]]] = field(
        default_factory=lambda: ["GET", "HEAD"]
    )
    path_pattern: Optional[str] = None
    cache_policy_id: Optional[str] = None
    compress: Optional[bool] = True
    default_ttl: Optional[int] = None
    field_level_encryption_id: Optional[str] = None
    forwarded_values: Optional[ForwardedValues] = None
    function_associations: Optional[List[FunctionAssociations]] = None
    max_ttl: Optional[int] = 0
    min_ttl: Optional[int] = 0
    origin_request_policy_id: Optional[str] = None
    realtime_log_config_arn: Optional[str] = None
    response_headers_policy_id: Optional[str] = None
    smooth_streaming: Optional[bool] = None
    trusted_key_groups: Optional[List[str]] = None
    trusted_signers: Optional[List[str]] = None

    def __post_init__(self):
        if not self.forwarded_values and not self.cache_policy_id:
            raise TheiaConfigurationError(
                "You must specify cache_policy_id or forwarded_values for default_cache_behavior"
            )
        if self.behaviour_type == "ordered" and not self.path_pattern:
            raise TheiaConfigurationError("You must specify path_pattern for ordered cache_behaviour")


@dataclass
class LoggingConfig:
    bucket: str
    include_cookies: Optional[bool] = False
    prefix: Optional[str] = None


@dataclass
class OriginGroups:
    failover_criteria_status_code: List[int]
    members_origin_id: List[str]
    origin_id: str


@dataclass
class CustomHeaders:
    name: str
    value: str


@dataclass
class CustomOriginConfig:
    http_port: int
    https_port: int
    origin_protocol_policy: Literal["http-only", "match-viewer", "https-only"]
    origin_ssl_protocols: Optional[List[Literal["SSLv3", "TLSv1", "TLSv1.1", "TLSv1.2"]]] = None
    origin_read_timeout: Optional[int] = None
    origin_keepalive_timeout: Optional[int] = None


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class Origins:
    domain_name: str
    origin_id: str
    connection_attempts: Optional[int] = 3
    connection_timeout: Optional[int] = 10
    custom_headers: Optional[List[CustomHeaders]] = None
    custom_origin_config: Optional[CustomOriginConfig] = None
    origin_access_control_id: Optional[str] = None
    origin_path: Optional[str] = None
    origin_shield_region: Optional[str] = None  # if yes set enabled true
    s3_origin_config_oai: Optional[Output[str] | str] = None


@dataclass
class Restrictions:
    restriction_type: Optional[Literal["none", "whitelist", "blacklist"]] = "none"
    locations: Optional[List[str]] = None

    def __post_init__(self):
        if (self.restriction_type == "whitelist" or self.restriction_type == "blacklist") and len(self.locations) == 0:
            raise TheiaConfigurationError(
                "You must specify locations when using whitelist or blacklist for restrictions"
            )


@dataclass
class ViewerCertificate:
    acm_certificate_arn: Optional[str] = None
    cloudfront_default_certificate: Optional[bool] = False
    iam_certificate_id: Optional[str] = None
    minimum_protocol_version: Optional[str] = "TLSv1"
    ssl_support_method: Optional[Literal["vip", "sni-only"]] = "sni-only"

    def __post_init__(self):
        if self.cloudfront_default_certificate == True and not self.acm_certificate_arn and not self.iam_certificate_id:
            raise TheiaConfigurationError(
                "You must specify acm_certificate_arn or iam_certificate_id when using cloudfront_default_certificate as true"
            )
        if not self.acm_certificate_arn and not self.iam_certificate_id:
            raise TheiaConfigurationError("You must specify acm_certificate_arn or iam_certificate_id")
        if self.cloudfront_default_certificate and self.minimum_protocol_version:
            raise TheiaConfigurationError(
                "You cannot specify minimum_protocol_version when using cloudfront_default_certificate as true"
            )
        minimum_protocol = ["TLSv1.2_2021", "TLSv1.2_2019", "TLSv1.2_2018", "TLSv1.1_2016", "TLSv1_2016", "TLSv1"]
        if (
            (self.acm_certificate_arn or self.iam_certificate_id)
            and self.ssl_support_method == "sni-only"
            and self.cloudfront_default_certificate == False
            and self.minimum_protocol_version not in minimum_protocol
        ):
            raise TheiaConfigurationError(
                "You must specify the minimum_protocol_version of the following "
                + minimum_protocol
                + " while using acm_certificate_arn or iam_certificate_id and setting cloudfront_default_certificate to false and ssl_support_method to sni-only"
            )
        if (
            (self.acm_certificate_arn or self.iam_certificate_id)
            and self.ssl_support_method == "vip"
            and self.cloudfront_default_certificate == False
            and (self.ssl_support_method != "SSLv3" or self.ssl_support_method != "TLSv1")
        ):
            raise TheiaConfigurationError(
                "You must specify the minimum_protocol_version of SSLv3 or TLSv1 while using acm_certificate_arn or iam_certificate_id and setting cloudfront_default_certificate to false and ssl_support_method to vip"
            )
        if self.cloudfront_default_certificate == True and self.minimum_protocol_version != "TSLv1":
            raise TheiaConfigurationError(
                "You must specify minimum_protocol_version of TSLv1 when using cloudfront_default_certificate as true"
            )


class Distribution(pulumi.ComponentResource):
    def __init__(
        self,
        resource_name: str,
        comment: Optional[str] = None,
        dist_settings: Optional[DistributionSettings] = DistributionSettings(),
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
        custom_error_responses_settings: Optional[List[CustomErrorResponses]] = None,
        logging_config_settings: Optional[LoggingConfig] = None,
        cache_behaviors_settings: Optional[List[CacheBehaviour]] = None,
        origin_groups_settings: Optional[List[OriginGroups]] = None,
        origins_settings: Optional[List[Origins]] = None,
        restrictions_settings: Optional[Restrictions] = Restrictions(),
        viewer_certificate_settings: Optional[ViewerCertificate] = None,
    ) -> None:
        super().__init__("theia:aws:network", resource_name, None, opts)
        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        depends = []
        logger.info(f"creating Cloudfront resource {resource_name}")
        logging_config = None
        if logging_config_settings:
            logging_config = cloudfront.DistributionLoggingConfigArgs(
                bucket=logging_config_settings.bucket,
                include_cookies=logging_config_settings.include_cookies,
                prefix=logging_config_settings.prefix,
            )
        custom_error_responses = []
        if custom_error_responses_settings:
            for error_reponse in custom_error_responses_settings:
                custom_error_responses.append(
                    cloudfront.DistributionCustomErrorResponseArgs(
                        error_code=error_reponse.error_code,
                        error_caching_min_ttl=error_reponse.error_caching_min_ttl,
                        response_code=error_reponse.response_code,
                        response_page_path=error_reponse.response_page_path,
                    )
                )
        count = 0
        cache_behaviour = []
        default_cache_behavior = None
        ordered_cache_behaviors = []
        if cache_behaviors_settings:
            for cache_behaviour in cache_behaviors_settings:
                if cache_behaviour.behaviour_type == "default":
                    count = count + 1
        if count > 1:
            raise TheiaConfigurationError("You cannot have more than one default cache behavior")
        elif count < 1:
            raise TheiaConfigurationError("You must have at least one default cache behavior")
        elif count == 1:
            for cache_behaviour in cache_behaviors_settings:
                function_associate = []
                lambda_associate = []
                count_1 = 0
                count_2 = 0
                if cache_behaviour.function_associations:
                    for function in cache_behaviour.function_associations:
                        if function.function_type == "cloudfront":
                            count_1 = count_1 + 1
                        elif function.function_type == "lambda":
                            count_2 = count_2 + 1
                    if count_1 > 2:
                        raise TheiaConfigurationError("You cannot have more than two cloudfront function associations")
                    elif count_2 > 4:
                        raise TheiaConfigurationError("You cannot have more than four lambda function associations")
                    elif count_1 <= 2 and count_2 <= 4:
                        for function in cache_behaviour.function_associations:
                            if function.function_type == "cloudfront":
                                function_associate.append(
                                    cloudfront.DistributionDefaultCacheBehaviorFunctionAssociationArgs(
                                        event_type=function.event_type,
                                        function_arn=function.function_arn,
                                    )
                                )
                            elif function.function_type == "lambda":
                                lambda_associate.append(
                                    cloudfront.DistributionOrderedCacheBehaviorLambdaFunctionAssociationArgs(
                                        event_type=function.event_type,
                                        lambda_arn=function.lambda_arn,
                                        include_body=function.include_body,
                                    )
                                )
                if cache_behaviour.behaviour_type == "default":
                    default_cache_behavior = cloudfront.DistributionDefaultCacheBehaviorArgs(
                        allowed_methods=cache_behaviour.allowed_methods,
                        cached_methods=cache_behaviour.cached_methods,
                        target_origin_id=cache_behaviour.target_origin_id,
                        viewer_protocol_policy=cache_behaviour.viewer_protocol_policy,
                        cache_policy_id=cache_behaviour.cache_policy_id,
                        compress=cache_behaviour.compress,
                        default_ttl=cache_behaviour.default_ttl,
                        field_level_encryption_id=cache_behaviour.field_level_encryption_id,
                        forwarded_values=(
                            None
                            if not cache_behaviour.forwarded_values
                            else cloudfront.DistributionDefaultCacheBehaviorForwardedValuesArgs(
                                cookies=cloudfront.DistributionDefaultCacheBehaviorForwardedValuesCookiesArgs(
                                    forward=cache_behaviour.forwarded_values.cookies.forward,
                                    whitelisted_names=cache_behaviour.forwarded_values.cookies.whitelisted_names,
                                ),
                                query_string=cache_behaviour.forwarded_values.query_string,
                                headers=cache_behaviour.forwarded_values.headers,
                                query_string_cache_keys=cache_behaviour.forwarded_values.query_string_cache_keys,
                            )
                        ),
                        function_associations=function_associate,
                        lambda_function_associations=lambda_associate,
                        max_ttl=cache_behaviour.max_ttl,
                        min_ttl=cache_behaviour.min_ttl,
                        origin_request_policy_id=cache_behaviour.origin_request_policy_id,
                        realtime_log_config_arn=cache_behaviour.realtime_log_config_arn,
                        response_headers_policy_id=cache_behaviour.response_headers_policy_id,
                        smooth_streaming=cache_behaviour.smooth_streaming,
                        trusted_key_groups=cache_behaviour.trusted_key_groups,
                        trusted_signers=cache_behaviour.trusted_signers,
                    )
                elif cache_behaviour.behaviour_type == "ordered":
                    ordered_cache_behaviors.append(
                        cloudfront.DistributionOrderedCacheBehaviorArgs(
                            allowed_methods=cache_behaviour.allowed_methods,
                            path_pattern=cache_behaviour.path_pattern,
                            cached_methods=cache_behaviour.cached_methods,
                            target_origin_id=cache_behaviour.target_origin_id,
                            viewer_protocol_policy=cache_behaviour.viewer_protocol_policy,
                            cache_policy_id=cache_behaviour.cache_policy_id,
                            compress=cache_behaviour.compress,
                            default_ttl=cache_behaviour.default_ttl,
                            field_level_encryption_id=cache_behaviour.field_level_encryption_id,
                            forwarded_values=(
                                None
                                if not cache_behaviour.forwarded_values
                                else cloudfront.DistributionOrderedCacheBehaviorForwardedValuesArgs(
                                    cookies=cloudfront.DistributionOrderedCacheBehaviorForwardedValuesCookiesArgs(
                                        forward=cache_behaviour.forwarded_values.cookies.forward,
                                        whitelisted_names=cache_behaviour.forwarded_values.cookies.whitelisted_names,
                                    ),
                                    query_string=cache_behaviour.forwarded_values.query_string,
                                    headers=cache_behaviour.forwarded_values.headers,
                                    query_string_cache_keys=cache_behaviour.forwarded_values.query_string_cache_keys,
                                )
                            ),
                            function_associations=function_associate,
                            lambda_function_associations=lambda_associate,
                            max_ttl=cache_behaviour.max_ttl,
                            min_ttl=cache_behaviour.min_ttl,
                            origin_request_policy_id=cache_behaviour.origin_request_policy_id,
                            realtime_log_config_arn=cache_behaviour.realtime_log_config_arn,
                            response_headers_policy_id=cache_behaviour.response_headers_policy_id,
                            smooth_streaming=cache_behaviour.smooth_streaming,
                            trusted_key_groups=cache_behaviour.trusted_key_groups,
                            trusted_signers=cache_behaviour.trusted_signers,
                        )
                    )
        origin_groups = None
        if origin_groups_settings:
            for og in origin_groups_settings:
                members_oid = None
                for mem in og.members_origin_id:
                    members_oid.append(
                        cloudfront.DistributionOriginGroupMemberArgs(
                            origin_id=mem,
                        )
                    )
                origin_groups.append(
                    cloudfront.DistributionOriginGroupArgs(
                        origin_id=og.origin_id,
                        failover_criteria=cloudfront.DistributionOriginGroupFailoverCriteriaArgs(
                            status_codes=og.failover_criteria_status_code,
                        ),
                        members=members_oid,
                    )
                )
        origins = []
        if origins_settings:
            for origin in origins_settings:
                cust_headers = None
                if origin.custom_headers:
                    for headers in origin.custom_headers:
                        cust_headers.append(
                            cloudfront.DistributionOriginCustomHeaderArgs(
                                name=headers.name,
                                value=headers.value,
                            )
                        )
                origins.append(
                    cloudfront.DistributionOriginArgs(
                        domain_name=origin.domain_name,
                        origin_id=origin.origin_id,
                        connection_attempts=origin.connection_attempts,
                        connection_timeout=origin.connection_timeout,
                        origin_access_control_id=origin.origin_access_control_id,
                        origin_path=origin.origin_path,
                        custom_origin_config=(
                            None
                            if not origin.custom_origin_config
                            else cloudfront.DistributionOriginCustomOriginConfigArgs(
                                http_port=origin.custom_origin_config.http_port,
                                https_port=origin.custom_origin_config.https_port,
                                origin_protocol_policy=origin.custom_origin_config.origin_protocol_policy,
                                origin_ssl_protocols=origin.custom_origin_config.origin_ssl_protocols,
                                origin_read_timeout=origin.custom_origin_config.origin_read_timeout,
                                origin_keepalive_timeout=origin.custom_origin_config.origin_keepalive_timeout,
                            )
                        ),
                        origin_shield=(
                            None
                            if not origin.origin_shield_region
                            else cloudfront.DistributionOriginOriginShieldArgs(
                                origin_shield_region=origin.origin_shield_region,
                                enabled=True if (origin.origin_shield_region) else False,
                            )
                        ),
                        s3_origin_config=(
                            None
                            if not origin.s3_origin_config_oai
                            else cloudfront.DistributionOriginS3OriginConfigArgs(
                                origin_access_identity=origin.s3_origin_config_oai,
                            )
                        ),
                        custom_headers=cust_headers,
                    )
                )
        restrictions = cloudfront.DistributionRestrictionsArgs(
            geo_restriction=cloudfront.DistributionRestrictionsGeoRestrictionArgs(
                restriction_type=restrictions_settings.restriction_type,
                locations=restrictions_settings.locations,
            )
        )
        viewer_certificate = None
        if viewer_certificate_settings:
            viewer_certificate = cloudfront.DistributionViewerCertificateArgs(
                acm_certificate_arn=viewer_certificate_settings.acm_certificate_arn,
                cloudfront_default_certificate=viewer_certificate_settings.cloudfront_default_certificate,
                iam_certificate_id=viewer_certificate_settings.iam_certificate_id,
                minimum_protocol_version=viewer_certificate_settings.minimum_protocol_version,
                ssl_support_method=viewer_certificate_settings.ssl_support_method,
            )
        distribution_args = cloudfront.DistributionArgs(
            aliases=dist_settings.aliases,
            comment=comment,
            custom_error_responses=custom_error_responses,
            default_cache_behavior=default_cache_behavior,
            default_root_object=dist_settings.default_root_object,
            enabled=True,
            http_version=dist_settings.http_version,
            is_ipv6_enabled=False,
            logging_config=logging_config,
            ordered_cache_behaviors=ordered_cache_behaviors,
            origin_groups=origin_groups,
            origins=origins,
            price_class=dist_settings.price_class,
            restrictions=restrictions,
            retain_on_delete=False,
            viewer_certificate=viewer_certificate,
            wait_for_deployment=True,
            web_acl_id=None,
            tags=merged_tags,
        )
        self.dist = cloudfront.Distribution(
            resource_name=f"{resource_name}",
            args=distribution_args,
            opts=ResourceOptions(parent=self, depends_on=depends),
        )
        pulumi.export(
            f"Cloudfront---{resource_name}",
            {
                "id": self.dist.id,
                "arn": self.dist.arn,
                "domain_name": self.dist.domain_name,
                "etag": self.dist.etag,
            },
        )
        self.register_outputs({})

    @staticmethod
    def import_(stack: str, resource_name: str) -> cloudfront.Distribution:
        stack_ref = StackRegistry.get(stack_name=stack)
        cdn = stack_ref.get_output(name=f"Cloudfront---{resource_name}")
        return cloudfront.Distribution.get(resource_name=resource_name, id=cdn["id"])


class OriginAccessIdentity(pulumi.ComponentResource):
    def __init__(
        self,
        resource_name: str,
        comment: Optional[str] = None,
        opts: Optional[ResourceOptions] = None,
    ) -> None:
        super().__init__("theia:aws:network", resource_name, None, opts)
        depends = []
        logger.info(f"creating Cloudfront OAI resource {resource_name}")
        oai_args = cloudfront.OriginAccessIdentityArgs(
            comment=comment,
        )
        self.oai_new = cloudfront.OriginAccessIdentity(
            resource_name=f"{resource_name}",
            args=oai_args,
            opts=ResourceOptions(parent=self, depends_on=depends),
        )
        pulumi.export(
            f"Cloudfront-OAI---{resource_name}",
            {
                "id": self.oai_new.id,
                "cloudfront_access_identity_path": self.oai_new.cloudfront_access_identity_path,
                "iam_arn": self.oai_new.iam_arn,
                "etag": self.oai_new.etag,
            },
        )

        self.register_outputs({})

    @staticmethod
    def import_(stack: str, resource_name: str) -> cloudfront.OriginAccessIdentity:
        stack_ref = StackRegistry.get(stack_name=stack)
        oai = stack_ref.get_output(name=f"Cloudfront-OAI---{resource_name}")
        return cloudfront.OriginAccessIdentity.get(resource_name=resource_name, id=oai["id"])


class CloudfrontFunction(pulumi.ComponentResource):
    def __init__(
        self,
        resource_name: str,
        opts: Optional[ResourceOptions] = None,
        code: Optional[str] = None,
        comment: Optional[str] = None,
        publish: Optional[bool] = None,
        runtime: Optional[str] = None,
    ) -> None:
        super().__init__("theia:aws:network", resource_name, None, opts)
        depends = []
        logger.info(f"creating Cloudfront Function resource {resource_name}")
        function_args = cloudfront.FunctionArgs(
            code=code,
            comment=comment,
            name=resource_name,
            publish=publish,
            runtime=runtime,
        )
        self.fun = cloudfront.Function(
            resource_name=f"{resource_name}",
            args=function_args,
            opts=ResourceOptions(parent=self, depends_on=depends),
        )
        pulumi.export(
            f"Cloudfront-Function---{resource_name}",
            {
                "id": self.fun.id,
                "arn": self.fun.arn,
                "etag": self.fun.etag,
                "status": self.fun.status,
                "live_stage_etag": self.fun.live_stage_etag,
            },
        )
        self.register_outputs({})

    @staticmethod
    def import_(stack: str, resource_name: str) -> cloudfront.Function:
        stack_ref = StackRegistry.get(stack_name=stack)
        function = stack_ref.get_output(name=f"Cloudfront-Function---{resource_name}")
        return cloudfront.Function.get(resource_name=resource_name, id=function["id"])


@dataclass
class CloudfrontPublicKeySettings:
    comment: Optional[str] = None


class CloudfrontPublicKey:
    def __new__(
        cls,
        resource_name: str,
        settings: Optional[CloudfrontPublicKeySettings] = CloudfrontPublicKeySettings(),
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
    ) -> cloudfront.PublicKey:
        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        private_key = RSA.generate(2048)
        pubkey = private_key.publickey()

        private_key = private_key.exportKey().decode("utf-8")
        pubkey = pubkey.exportKey().decode("utf-8")

        public_key = cloudfront.PublicKey(
            resource_name,
            name=resource_name,
            comment=settings.comment,
            encoded_key=pubkey,
            args=cloudfront.PublicKeyArgs(name=resource_name, encoded_key=pubkey),
            opts=ResourceOptions(ignore_changes=["encodedKey"]),
        )

        secret_name = f"{resource_name}-cloudfront-private-key"
        pk = secretsmanager.Secret(
            resource_name=secret_name,
            args=secretsmanager.SecretArgs(
                name=secret_name,
                tags=merged_tags,
            ),
            opts=ResourceOptions(ignore_changes=["encodedKey"]),
        )

        pk_secret = secretsmanager.SecretVersion(
            resource_name=secret_name,
            secret_id=pk.arn,
            secret_string=private_key,
            opts=ResourceOptions(parent=pk, depends_on=pk, ignore_changes=["secretString"]),
        )

        export_values = {
            "public_key_id": public_key.id,
            "private_key_secret": pk_secret.arn,
        }

        pulumi.export(f"cloudfront-kep-pair---{resource_name}", export_values)

        return public_key

    @staticmethod
    def import_(stack: str, resource_name: str) -> cloudfront.PublicKey:
        stack_ref = StackRegistry.get(stack_name=stack)
        kp = stack_ref.get_output(name=f"cloudfront-kep-pair---{resource_name}")
        return cloudfront.PublicKey.get(resource_name=resource_name, id=kp["id"])
