import pulumi

from pulumi_aws import ec2
from pulumi import Output

from pydantic.dataclasses import dataclass, ConfigDict
from typing import Optional, List, Mapping

from theia import ResourceOptions

from theia.definitions import THEIA_VERSION
from theia._registry import StackRegistry, GlobalTags


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class NetworkDHCPOptionsSettings:
    domain_name: Optional[str] = None
    domain_name_servers: Optional[List[str]] = None
    netbios_name_servers: Optional[List[str]] = None
    netbios_node_type: Optional[str] = None
    ntp_servers: Optional[List[str]] = None
    vpc_id: Optional[str | Output[str]] = None


class NetworkDHCPOptions:
    def __new__(
        cls,
        resource_name: str,
        settings: Optional[NetworkDHCPOptionsSettings] = None,
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
    ) -> ec2.VpcDhcpOptions:

        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        option_set = ec2.VpcDhcpOptions(
            resource_name=resource_name,
            domain_name=settings.domain_name,
            domain_name_servers=settings.domain_name_servers,
            netbios_name_servers=settings.netbios_name_servers,
            netbios_node_type=settings.netbios_node_type,
            ntp_servers=settings.ntp_servers,
            tags=merged_tags,
            opts=opts,
        )

        if settings.vpc_id:
            ec2.VpcDhcpOptionsAssociation(
                resource_name=f"{resource_name}-assc",
                vpc_id=settings.vpc_id,
                dhcp_options_id=option_set.id,
            )

        pulumi.export(f"dhcp-option-set---{resource_name}", {"id": option_set.id, "arn": option_set.arn})

        return option_set

    @staticmethod
    def import_(stack: str, resource_name: str) -> ec2.VpcDhcpOptions:
        stack_ref = StackRegistry.get(stack_name=stack)
        option_set = stack_ref.get_output(name=f"dhcp-option-set---{resource_name}")
        return ec2.VpcDhcpOptions.get(resource_name=resource_name, id=option_set["id"])
