import pulumi

from pulumi_aws import ec2
from pulumi.output import Output

from typing import Optional, Mapping
from pydantic.dataclasses import dataclass, ConfigDict

from theia import ResourceOptions
from theia.utils.logger import Logger
from theia.definitions import THEIA_VERSION
from theia._registry import StackRegistry, GlobalTags

logger = Logger("eip")


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class EipSettings:
    instance: Optional[str] = None
    network_interface: Optional[str | Output[str]] = None
    address: Optional[str | Output[str]] = None


class Eip:
    def __new__(
        cls,
        resource_name: str,
        tags: Optional[Mapping[str, str]] = None,
        settings: Optional[EipSettings] = EipSettings(),
        opts: Optional[ResourceOptions] = None,
    ) -> ec2.Eip:
        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        eip_args = ec2.EipArgs(
            address=settings.address,
            network_interface=settings.network_interface,
            instance=settings.instance,
            tags=merged_tags,
        )

        eip = ec2.Eip(resource_name, args=eip_args, opts=opts)
        export_values = {
            "id": eip.id,
            "allocation_id": eip.allocation_id,
            "private_dns": eip.private_dns,
            "private_ip": eip.private_ip,
            "public_dns": eip.public_dns,
            "public_ip": eip.public_ip,
        }

        pulumi.export(f"eip---{resource_name}", export_values)

        return eip

    @staticmethod
    def import_(stack: str, resource_name: str) -> ec2.Eip:
        stack_ref = StackRegistry.get(stack_name=stack)
        eip = stack_ref.get_output(name=f"eip---{resource_name}")
        return ec2.Eip.get(resource_name=resource_name, id=eip["id"])
